#!/bin/bash

# Note 1: Run tail -f /opt/zimbra/log/mailbox.log while running the zmmailbox command to see errors.
# Note 2: Empty folders will not be restored
# Note 3: EML files that have no email content, missing headers/body are not imported

WHO=`whoami`
if [ $WHO != "zimbra" ]
then
echo
echo "Execute this scipt as user zimbra (\"su - zimbra\")"
echo
exit 1
fi

if [ $# -ne 2 ]
then
echo ""
echo "usage: ./import-tgz.sh tgz mailbox"
echo "example: ./import-tgz.sh /path/to/import.tgz admin@example.com"
echo ""
exit 1
fi

if ! command -v screen &> /dev/null; then
   echo "Error: 'screen' command is not available."
   echo "run as root: apt-get install screen"
   exit 1
fi

if screen -ls | grep -q '\<zmmailbox\>'; then
   echo "Error: Screen session with the name 'zmmailbox' is already running."
   echo "To force terminate every screen session run as user zimbra: killall screen"
   exit 1
fi

#https://community.zextras.com/forum/zimbra-ose-9-by-zextras/zmmailbox-postresturl-doesnt-work/
zmlocalconfig -e socket_so_timeout=3000000

# Define a function to send the command to the screen session
send_command() {
   echo "<item>" >> /tmp/zmmailbox-screen/output.log 
   echo "Executing command:" >> /tmp/zmmailbox-screen/output.log
   echo "$1" >> /tmp/zmmailbox-screen/output.log
   screen -S zmmailbox -X stuff "$1\n"
   check_and_move_log
}

# Function to check if the mbox> prompt is in the monitor log, use to log and wait until zmmailbox is ready
check_and_move_log() {
   # Wait for the mbox> prompt to appear in the monitor log
   until grep -Eq "mbox( .*|)>" /tmp/zmmailbox-screen/monitor.log; do
      sleep 0.1
   done

   # Move the monitor log contents to the output log
   cat /tmp/zmmailbox-screen/monitor.log >> /tmp/zmmailbox-screen/output.log
   echo "</item>" >> /tmp/zmmailbox-screen/output.log
   echo "" > /tmp/zmmailbox-screen/monitor.log
}

create_tgz_files() {
   local dir="$1"

   # Find all .eml files in the directory and its subdirectories
   find "$dir" -type f -name "*.eml" | while IFS= read -r eml_file; do
      # Get the directory containing the .eml file
      eml_dir=$(dirname "$eml_file")
      
      # Get the relative path of the .eml file with respect to the starting directory
      rel_path="${eml_dir#$dir/}"
      
      # Create a .tgz file for the .eml file while retaining the folder structure
      tar -czf "${eml_file}.tgz" "${eml_file}" --transform='s/tmp\/tgz-extracted\///g'
   done
}

# Convert large tgz into many small tgz files
input_file="$1"
rm -Rf "/tmp/tgz-extracted/"
mkdir "/tmp/tgz-extracted/"
echo "Extracting source tgz file..."
tar -xzf "$input_file" -C "/tmp/tgz-extracted/"
echo "Generating new tgz files..."
create_tgz_files "/tmp/tgz-extracted"

# Prepare zmmailbox
rm -Rf /tmp/zmmailbox-screen
mkdir /tmp/zmmailbox-screen
cd /tmp/zmmailbox-screen
echo "<item>" > /tmp/zmmailbox-screen/output.log 
screen -L -Logfile /tmp/zmmailbox-screen/monitor.log  -d -m -S zmmailbox /opt/zimbra/bin/zmmailbox -t 0 -z -m $2
screen -S zmmailbox -X logfile flush 0
check_and_move_log

# This is just a test
# send_command "help"

echo "Commencing import into Zimbra"
echo "This can take a long time"
echo "Run the following command in a separate window to track progress:"
echo "tail -f /tmp/zmmailbox-screen/output.log"
echo "tail -f /opt/zimbra/log/mailbox.log | grep -i error | grep \"ParsedMessage\""

while IFS= read -r -d '' file; do
   #send_command "postRestURL '/?fmt=tgz&subfolder=import&timestamp=0&resolve=skip' '$file'\n"
   send_command "postRestURL '/?fmt=tgz&timestamp=0&resolve=skip' '$file'\n"
done < <(find "/tmp/tgz-extracted/" -type f -name "*.tgz" -print0)

screen -S zmmailbox -X quit

zmlocalconfig -u socket_so_timeout

echo "EML files and other data remain in /tmp/tgz-extracted and /tmp/zmmailbox-screen"
echo "Review if needed and delete manually. Re-running this script will remove previously stored data."
