# .\pst-to-tgz.ps1 "C:\Users\user\Downloads\rogers b.pst"

# PSTExtractor creates eml files based on Message-ID, but Zimbra REST import does not like that, so we renumber them in each folder
# similar to 
function Rename-Files {
   param (
     [string]$Directory
   )

   $count = 0

   # Get all files in the directory
   $files = Get-ChildItem -Path $Directory -File -Recurse

   foreach ($file in $files) {
      if ($file.Extension -eq ".eml") {
         $count++
         # Rename the file with the count
         $newName = "$($count)$($file.Extension)"
         $file | Rename-Item -NewName $newName
       }
   }
}

# PSTExtractor creates .meta files with metadata, but these can not be used by Zimbra server, so we remove them
function Remove-MetaFiles {
   param (
      [string]$Directory
   )

   # Get all .meta files in the directory
   $metaFiles = Get-ChildItem -Path $Directory -Filter *.meta -File -Recurse

   # Remove each .meta file
   foreach ($file in $metaFiles) {
      Remove-Item -Path $file.FullName -Force
   }
}

# Alternative open source tool: readpst -M -b -q -e -o export rogers-b.pst may not support all versions of outlook, this is why we use:
$exePath = "C:\Program Files\Zimbra Desktop\resources\PSTExtractor\PSTExtractor.exe"

if (-not (Test-Path -Path $exePath)) {
   Write-Host "This application requires Zimbra Desktop to be installed."
   exit
}

$pstfile=$args[0]
if (-not (Test-Path -Path $pstfile)) {
   Write-Host 'This script requires the first argument to point to a .pst file, example usage: .\pst-to-tgz.ps1 "C:\Users\user\Downloads\rogers b.pst"'
   exit
}

Remove-Item -ErrorAction SilentlyContinue -Recurse -Force "c:\tmp\pst-to-tgz"
mkdir "c:\tmp\pst-to-tgz"

write-host "Extracting PST file..."
& $exePath input=$pstfile output="c:\tmp\pst-to-tgz\export" logpath="c:\tmp\pst-to-tgz\log"

$sourceDirectory = "C:\tmp\pst-to-tgz\export\"

write-host "Removing .meta files..."
Remove-MetaFiles -Directory $sourceDirectory

write-host "Renaming files..."
Rename-Files -Directory $sourceDirectory


write-host "Creating tgz..."
# Get the list of directories inside the export folder
$subDirectories = Get-ChildItem -Path $sourceDirectory -Directory

if ($subDirectories.Count -eq 0) {
   Write-Host "No subdirectories found in $sourceDirectory"
}
else {
   # Assume there's only one subdirectory
   $folderName = $subDirectories[0].Name

   # Set the path to the folder you want to compress
   $folderToCompress = Join-Path -Path $sourceDirectory -ChildPath $folderName

   # Set the output file path
   $outputFilePath = $pstfile + ".tgz"

   tar -czvf $outputFilePath -C $sourceDirectory $folderName

   Write-Host "Compression complete. Output file: $outputFilePath"
}

write-host "Please note that logs and eml export remain in C:\tmp\pst-to-tgz, please remove after manual verification."
write-host "To import this tgz file use the import-tgz.sh script, do not attempt to load via the WebUI!"
